package salaComunitaria.controllers;

import java.util.List;

import jakarta.inject.Inject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Request;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;
import salaComunitaria.entities.Event;
import salaComunitaria.interfacesDAO.EventDAO;

@Path("/events")
public class EventController {
	
	@Context
	UriInfo uriInfo;

	@Context
	Request request;
	
    @Context
    HttpServletRequest httpRequest;
    
    @Inject
    EventDAO eventDAO;
    
    private String message;
    
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response getEvents() {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		List<Event> events = eventDAO.get();
		
		return Response.ok().entity(events).build();
	}
	
	@GET
	@Path("/getBy/{id}")
	@Produces(MediaType.APPLICATION_JSON)
	public Response find(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		Event event = eventDAO.get(id);
		
		return Response.ok().entity(event).build();
		
	}
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create (Event event) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		if(event != null) {
			eventDAO.create(event);
			return Response.status(Response.Status.CREATED).build();
		} else {
			return Response.status(Response.Status.CONFLICT).build();
		}
		
		
	}

}
