package salaComunitaria.controllers;

import java.util.List;
import java.util.stream.Collectors;

import jakarta.inject.Inject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Request;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;
import salaComunitaria.entities.RawMaterial;
import salaComunitaria.interfacesDAO.RawMaterialDAO;

@Path("/RawMaterials")
public class RawMaterialController {

	@Context
	UriInfo uriInfo;

	@Context
	Request request;
	
    @Context
    HttpServletRequest httpRequest;
	
	@Inject
	private RawMaterialDAO rawmaterialDao;
	private String message;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response  getProviders() {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		List<RawMaterial> rawMaterials = rawmaterialDao.get();
		
		return Response.ok().entity(rawMaterials).build();
	}
	
	@GET
	@Path("/getBy/{id}")
	@Produces(MediaType.APPLICATION_JSON)
	public Response find(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		RawMaterial rawMaterial = rawmaterialDao.get(id);
		if (rawMaterial != null) {
			return Response.ok().entity(rawMaterial).build();
		} else {
			message = "No se encontró la materia prima";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}

	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create(RawMaterial rawMaterial) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		if (rawMaterial != null) { 
			rawmaterialDao.saveRawMaterial(rawMaterial);
			return Response.status(Response.Status.CREATED).build();
		} else {
			return Response.status(Response.Status.CONFLICT).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response modify(RawMaterial rawMaterial) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		RawMaterial aux = rawmaterialDao.get(rawMaterial.getId());
		if (aux != null) {
			rawmaterialDao.modify(rawMaterial);
			return Response.ok().entity(rawMaterial).build();
		} else {
			return Response.status(Response.Status.NOT_FOUND).entity("[]").build();
		}
	}

	@DELETE
	@Path("{id}")
	@Produces(MediaType.TEXT_PLAIN)
	public Response delete(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		RawMaterial aux = rawmaterialDao.get(id);
		if (aux != null) {
			rawmaterialDao.delete(aux);
			return Response.noContent().build();
		} else {
			message = "No existe materia prima con ese id";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}
	
	@GET
	@Path("/getTypes")
	@Produces(MediaType.APPLICATION_JSON)
	public Response getTypes() {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		List<RawMaterial> rawMaterials = rawmaterialDao.get();
		
		List<String> types = rawMaterials.stream()
				.map( i -> i.getType())
				.distinct()
				.sorted()
				.collect(Collectors.toList());
		
		return Response.ok().entity(types).build();
	
	}
}
