package salaComunitaria.controllers;

import java.util.List;

import jakarta.inject.Inject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Request;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;
import salaComunitaria.entities.Recipe;
import salaComunitaria.interfacesDAO.RecipeDAO;

@Path("/Recipes")
public class RecipeController {
	
	@Context
	UriInfo uriInfo;

	@Context
	Request request;
	
    @Context
    HttpServletRequest httpRequest;
	
	@Inject
	private RecipeDAO recipeDao;
	private String message;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response getRecipe() {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		List<Recipe> recipes = recipeDao.get();
		
		return Response.ok().entity(recipes).build();
	}
	
	@GET
	@Path("/{id}")
	@Produces(MediaType.APPLICATION_JSON)
	public Response find(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		Recipe recipe = recipeDao.get(id);
		if (recipe != null) {
			return Response.ok().entity(recipe).build();
		} else {
			message = "No se encontró la receta";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}

	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create(Recipe recipe) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		if (recipe != null) { 
			recipeDao.create(recipe);
			return Response.status(Response.Status.CREATED).build();
		} else {
			return Response.status(Response.Status.CONFLICT).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response modify(Recipe recipe) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		Recipe aux = recipeDao.get(recipe.getId());
		if (aux != null) {
			recipeDao.modify(recipe);
			return Response.ok().entity(recipe).build();
		} else {
			return Response.status(Response.Status.NOT_FOUND).entity("[]").build();
		}
	}

	@DELETE
	@Path("{id}")
	@Produces(MediaType.TEXT_PLAIN)
	public Response delete(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		Recipe aux = recipeDao.get(id);
		if (aux != null) {
			recipeDao.delete(aux);
			return Response.noContent().build();
		} else {
			message = "No existe receta con ese id";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}
}
