package salaComunitaria.controllers;

import java.util.List;

import jakarta.inject.Inject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Request;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;
import salaComunitaria.entities.SalesChannel;
import salaComunitaria.interfacesDAO.SalesChannelDAO;

@Path("/SalesChannels")
public class SalesChannelController {
	
	@Context
	UriInfo uriInfo;

	@Context
	Request request;
	
    @Context
    HttpServletRequest httpRequest;
	
	@Inject
	private SalesChannelDAO salesChannelDao;
	private String message;
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response getSalesChannels() {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		List<SalesChannel> saleschannels = salesChannelDao.get();
		
		return Response.ok().entity(saleschannels).build();
	}
	
	@GET
	@Path("/getBy/{id}")
	@Produces(MediaType.APPLICATION_JSON)
	public Response find(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		SalesChannel salesChannel = salesChannelDao.get(id);
		if (salesChannel != null) {
			return Response.ok().entity(salesChannel).build();
		} else {
			message = "No se encontró el Canal de Venta";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}

	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create(SalesChannel salesChannel) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		if (salesChannel != null) { 
			salesChannelDao.create(salesChannel);
			return Response.status(Response.Status.CREATED).build();
		} else {
			return Response.status(Response.Status.CONFLICT).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response modify(SalesChannel salesChannel) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		SalesChannel aux = salesChannelDao.get(salesChannel.getId());
		if (aux != null) {
			salesChannelDao.modify(salesChannel);
			return Response.ok().entity(salesChannel).build();
		} else {
			return Response.status(Response.Status.NOT_FOUND).entity("[]").build();
		}
	}

	@DELETE
	@Path("{id}")
	@Produces(MediaType.TEXT_PLAIN)
	public Response delete(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		SalesChannel aux = salesChannelDao.get(id);
		if (aux != null) {
			salesChannelDao.delete(aux);
			return Response.noContent().build();
		} else {
			message = "No existe Canal de Venta con ese id";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}
}
