package salaComunitaria.controllers;

import java.util.List;

import jakarta.inject.Inject;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Request;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;
import salaComunitaria.entities.User;
import salaComunitaria.exceptions.DuplicateKeyException;
import salaComunitaria.interfacesDAO.UserDAO;

@Path("/user")
public class UserController {

	@Context
	UriInfo uriInfo;

	@Context
	Request request;
	
    @Context
    HttpServletRequest httpRequest;
	
	@Inject
	private UserDAO udao;
	private String message;

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response  getUsuarios() {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		List<User> users = udao.get();
		return Response.ok().entity(users).build();
	}

	@GET
	@Path("/{id}")
	@Produces(MediaType.APPLICATION_JSON)
	public Response find(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		User user = udao.get(id);
		if (user != null) {
			return Response.ok().entity(user).build();
		} else {
			message = "No se encontró el usuario";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}

	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create(User usuario) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		if (usuario != null) {
			 
			try{
				udao.saveUser(usuario);
			return Response.status(Response.Status.CREATED).build();
			} catch(DuplicateKeyException e) {
				return Response.status(Response.Status.CONFLICT).entity(e.getMessage()).build();
			}
			
			
		} else {
			return Response.status(Response.Status.CONFLICT).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response modify(User usuario) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		User aux = udao.get(usuario.getId());
		if (aux != null) {
			udao.modify(usuario);			;
			return Response.ok().entity(usuario).build();
		} else {
			return Response.status(Response.Status.NOT_FOUND).entity("[]").build();
		}
	}

	@DELETE
	@Path("{id}")
	@Produces(MediaType.TEXT_PLAIN)
	public Response delete(@PathParam("id") Integer id) {
		
		String tokenStatus = (String) httpRequest.getAttribute("token");
		
		if(tokenStatus != "ok") {
			message = "Token JWT invalido";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
		
		User aux = udao.get(id);
		if (aux != null) {
			udao.delete(aux);
			return Response.noContent().build();
		} else {
			message = "No existe el usuario con ese id";
			return Response.status(Response.Status.NOT_FOUND).entity(message).build();
		}
	}

}
