package salaComunitaria.entities;

import java.util.Date;
import java.util.Set;

import javax.persistence.*;

import org.hibernate.annotations.ResultCheckStyle;
import org.hibernate.annotations.SQLDelete;
import org.hibernate.annotations.Where;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@SQLDelete(sql = "UPDATE INPUTS SET state = 'DELETED' WHERE INPUT_ID = ?", check = ResultCheckStyle.COUNT)
@Where(clause = "state <> 'DELETED'")
@NamedQuery(name = "inputs.FindById", query = "SELECT a FROM Input a WHERE a.id = :id")
@Table(name = "INPUTS")
public class Input {

	// Attributes
	@Id
	@GeneratedValue
	@Column(name = "INPUT_ID")
	private long id;

	@Column(nullable = false)
	private String name;

	@Column(nullable = false)
	private String type;

	@Column(nullable = false)
	private String unit;

	@Column(nullable = false)
	private double quantity;

	@Column(nullable = false)
	private double cost;

	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date admissionDate;

	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date lastModificationDate;

	@Column(nullable = false)
	private String description;
	
	//Los estados que se van a manejar son ACTIVE y DELETED
    @Column(nullable=false, name="state")
    private String state = "ACTIVE";

    //Relations
	@ManyToOne
	@JoinColumn(name = "PROVIDER_ID")
	private Provider provider;

	@JsonIgnore
	@ManyToMany(mappedBy = "batchPackagingElements", cascade = { CascadeType.PERSIST, CascadeType.MERGE })
	private Set<Batch> batches;

	@PrePersist
	protected void onCreate() {
		admissionDate = new Date(); // Fecha actual cuando se crea la entidad
		lastModificationDate = new Date(); // Fecha actual cuando se crea la entidad
	}

	@PreUpdate
	protected void onUpdate() {
		lastModificationDate = new Date();
	}

	public Input() {
		super();
	}

	// Getters and Setters
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getUnit() {
		return unit;
	}

	public void setUnit(String unit) {
		this.unit = unit;
	}

	public double getQuantity() {
		return quantity;
	}

	public void setQuantity(double quantity) {
		this.quantity = quantity;
	}

	public double getCost() {
		return cost;
	}

	public void setCost(double cost) {
		this.cost = cost;
	}

	public Date getAdmissionDate() {
		return admissionDate;
	}

	public void setAdmissionDate(Date admissionDate) {
		this.admissionDate = admissionDate;
	}

	public Date getLastModificationDate() {
		return lastModificationDate;
	}

	public void setLastModificationDate(Date lastModificationDate) {
		this.lastModificationDate = lastModificationDate;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Provider getProvider() {
		return provider;
	}

	public void setProvider(Provider provider) {
		this.provider = provider;
	}

	public Set<Batch> getBatches() {
		return batches;
	}

	public void setBatches(Set<Batch> batches) {
		this.batches = batches;
	}

	@Override
	public String toString() {
		return "Input [id=" + id + ", name=" + name + ", type=" + type + ", unit=" + unit + ", quantity=" + quantity
				+ ", cost=" + cost + ", admissionDate=" + admissionDate + ", lastModificationDate="
				+ lastModificationDate + ", description=" + description + ", provider=" + provider + "]";
	}

}
