package salaComunitaria.entities;

import java.util.Set;

import javax.persistence.*;

import org.hibernate.annotations.ResultCheckStyle;
import org.hibernate.annotations.SQLDelete;
import org.hibernate.annotations.Where;

import com.fasterxml.jackson.annotation.JsonIgnore;


@Entity
@SQLDelete(sql = "UPDATE PROVIDERS SET state = 'DELETED' WHERE PROVIDER_ID = ?", check = ResultCheckStyle.COUNT)
@Where(clause = "state <> 'DELETED'")
@NamedQuery(name = "providers.FindById", query = "SELECT a FROM Provider a WHERE a.id = :id")
@Table(name="PROVIDERS")
public class Provider {
	
	//Attributes
	@Id @GeneratedValue
	@Column(name="PROVIDER_ID")
	private long id;
	
	@Column(nullable=false)
	private String cuit;
	
	@Column(nullable=false)
	private String name;
	
	@Column(nullable=false)
	private String description;
	
	@Column(nullable=false)
	private String email;
	
	@Column(nullable=false)
	private String phone;
	
	@Column(nullable=false)
	private String address;
	
	//Los estados que se van a manejar son ACTIVE y DELETED
    @Column(nullable=false, name="state")
    private String state = "ACTIVE";
	
	//Relations
	@JsonIgnore
	@OneToMany(mappedBy = "provider")
	private Set<RawMaterial> rawMaterials;
	
	@JsonIgnore
	@OneToMany(mappedBy = "provider")
	private Set<Input> inputs;
	
	public Provider() {
		super();
	}

	//Getters and Setters
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}
	
	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getCuit() {
		return cuit;
	}

	public void setCuit(String cuit) {
		this.cuit = cuit;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public Set<RawMaterial> getRawMaterials() {
		return rawMaterials;
	}

	public void setRawMaterials(Set<RawMaterial> rawMaterials) {
		this.rawMaterials = rawMaterials;
	}

	public Set<Input> getInputs() {
		return inputs;
	}

	public void setInputs(Set<Input> inputs) {
		this.inputs = inputs;
	}

	@Override
	public String toString() {
		return "Provider [id=" + id + ", cuit=" + cuit + ", name=" + name + ", description=" + description + ", email="
				+ email + ", phone=" + phone + ", address=" + address + "]";
	}
	
	
}
