package salaComunitaria.entities;

import java.util.Date;
import java.util.Set;

import javax.persistence.*;

import org.hibernate.annotations.ResultCheckStyle;
import org.hibernate.annotations.SQLDelete;
import org.hibernate.annotations.Where;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@SQLDelete(sql = "UPDATE RAW_MATERIALS SET state = 'DELETED' WHERE RAW_MATERIAL_ID = ?", check = ResultCheckStyle.COUNT)
@Where(clause = "state <> 'DELETED'")
@NamedQuery(name = "raw_materials.FindById", query = "SELECT a FROM RawMaterial a WHERE a.id = :id")
@Table(name="RAW_MATERIALS")
public class RawMaterial {
	
	//Attributes
	@Id @GeneratedValue
	@Column(name="RAW_MATERIAL_ID")
	private long id;
	
	@Column(nullable=false)
	private String name;

	@Column(nullable=false)
	private double quantity;
	
	@Column(nullable=false)
	private Boolean freeze;
	
	@Column(nullable=false)
	private double cost;
	
	@Column
	@Temporal(TemporalType.TIMESTAMP)
	private Date expirationDate;
	
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date admissionDate;
	
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date lastModificationDate;
	
	@Column(nullable=false)
	private String descripction;
	
	@Column(nullable=false)
	private String unit;
	
	@Column(nullable=false)
	private String type;
	
	//Los estados que se van a manejar son ACTIVE y DELETED
    @Column(nullable=false, name="state")
    private String state = "ACTIVE";
	
	//Relations
	//@JsonIgnore
	@ManyToOne
	@JoinColumn(name = "PROVIDER_ID")
	private Provider provider;
	
	@JsonIgnore
	@ManyToMany(mappedBy="batchIngredients")
	private Set<Batch> batches;
	
	
	@PrePersist
	protected void onCreate() {
		admissionDate = new Date(); // Fecha actual cuando se crea la entidad
		lastModificationDate = new Date(); // Fecha actual cuando se crea la entidad
	}

	@PreUpdate
	protected void onUpdate() {
		lastModificationDate = new Date();
	}
	
	public RawMaterial() {
		super();
	}


	//Getters and Setters
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}
	
	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public double getQuantity() {
		return quantity;
	}

	public void setQuantity(double quantity) {
		this.quantity = quantity;
	}

	public Boolean getFreeze() {
		return freeze;
	}

	public void setFreeze(Boolean freeze) {
		this.freeze = freeze;
	}

	public double getCost() {
		return cost;
	}

	public void setCost(double cost) {
		this.cost = cost;
	}

	public Date getExpirationDate() {
		return expirationDate;
	}

	public void setExpirationDate(Date expirationDate) {
		this.expirationDate = expirationDate;
	}

	public Date getAdmissionDate() {
		return admissionDate;
	}

	public void setAdmissionDate(Date admissionDate) {
		this.admissionDate = admissionDate;
	}

	public Date getLastModificationDate() {
		return lastModificationDate;
	}

	public void setLastModificationDate(Date lastModificationDate) {
		this.lastModificationDate = lastModificationDate;
	}

	public String getDescripction() {
		return descripction;
	}

	public void setDescripction(String descripction) {
		this.descripction = descripction;
	}

	public String getUnit() {
		return unit;
	}

	public void setUnit(String unit) {
		this.unit = unit;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public Provider getProvider() {
		return provider;
	}

	public void setProvider(Provider provider) {
		this.provider = provider;
	}

	public Set<Batch> getBatches() {
		return batches;
	}

	public void setBatches(Set<Batch> batches) {
		this.batches = batches;
	}
		
	
	public String getName() {
		return name;
	}


	public void setName(String name) {
		this.name = name;
	}


	@Override
	public String toString() {
		return "RawMaterial [id=" + id + ", name=" + name + ", quantity=" + quantity + ", freeze=" + freeze + ", cost="
				+ cost + ", expirationDate=" + expirationDate + ", admissionDate=" + admissionDate
				+ ", lastModificationDate=" + lastModificationDate + ", descripction=" + descripction + ", unit=" + unit
				+ ", type=" + type + ", provider=" + provider + "]";
	}

}
