package salaComunitaria.objectsDAO;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.NoResultException;
import javax.persistence.Persistence;
import javax.persistence.Query;

import jakarta.annotation.PreDestroy;
import jakarta.inject.Inject;


public class GenericDAOImpl<T> extends GenericDAOAbs<T> {
	
	@Inject
	private EntityManager em;

	private static EntityManagerFactory emf;
	
	public static EntityManagerFactory getEmf() {
		return emf;
	}

	public static void setEmf(EntityManagerFactory emf) {
		GenericDAOImpl.emf = emf;
	}

	public GenericDAOImpl() {
		super();
		if (emf == null) {
			emf = Persistence.createEntityManagerFactory("miUP");
		}
	}
	
	@PreDestroy
	public void closeConnection() {
		em.close();
	}

	@Override
	public void create(T entity) {
		EntityTransaction etx = null;
		try{
			em = emf.createEntityManager();
			etx= em.getTransaction();
			etx.begin();
			em.merge(entity); 
			etx.commit();
		} catch(Exception e) {
			if(etx  != null && etx.isActive())
				etx.rollback();
			e.printStackTrace();
		}finally {
			em.close();
		}
	}

	@Override
	public void delete(T entity) {
		EntityTransaction etx = null;
		try {
			em = emf.createEntityManager();
			etx = em.getTransaction();
			etx.begin();
				
			em.remove(em.contains(entity) ? entity : em.merge(entity));
			etx.commit();
			
		} catch(Exception e) {
			if(etx  != null && etx.isActive())
				etx.rollback();
			e.printStackTrace();
		}finally {
			em.close();
		}
	}

	@Override
	public void modify(T entity) {
		EntityTransaction etx = null;
		try {
			em = emf.createEntityManager();
			etx = em.getTransaction();
			etx.begin();
			em.merge(entity);
			etx.commit();
		} catch(Exception e) {
			if(etx  != null && etx.isActive())
				etx.rollback();
			e.printStackTrace();		
		}finally {
			em.close();
		}
	}

	@Override
	public List<T> get() {
		List<T> entities = null;
		try {
			em = emf.createEntityManager();
			String fullClassName = classType.getTypeName();
			int lastDotIndex = fullClassName.lastIndexOf('.');
			String simpleClassName = fullClassName.substring(lastDotIndex + 1);
			
			
			String queryString = "SELECT e FROM " + simpleClassName + " e";
			Query query = em.createQuery(queryString, classType.getClass().forName(classType.getTypeName()));
			entities = query.getResultList();
			entities.size();
		}catch(Exception e) {
            e.printStackTrace();
		}
//		}finally {
//			em.close();
//		}
		return entities;
	}

	@Override
	public T get(long id) {
		T entity = null;
		try {
			em=emf.createEntityManager();
			String fullClassName = classType.getTypeName();
			int lastDotIndex = fullClassName.lastIndexOf('.');
			String simpleClassName = fullClassName.substring(lastDotIndex + 1);
			
			String queryString = "SELECT e FROM " + simpleClassName + " e WHERE e.id = :id";
			Query query = em.createQuery(queryString, classType.getClass().forName(classType.getTypeName()));
		    query.setParameter("id", id);
			entity = (T) query.getSingleResult();
		} catch (NoResultException e) {
			e.printStackTrace();	
		}catch(Exception e) {
			e.printStackTrace();
		}
//		}finally {
//			em.close();
//		}
		return entity;
		
	}

	public static void closeEntityManagerFactory() {
		if(emf != null)
			emf.close();
	}

}
