package salaComunitaria.objectsDAO;

import java.sql.SQLException;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.PersistenceException;

import org.jvnet.hk2.annotations.Service;

import jakarta.validation.ConstraintViolationException;
import salaComunitaria.entities.Event;
import salaComunitaria.entities.User;
import salaComunitaria.exceptions.DuplicateKeyException;
import salaComunitaria.interfacesDAO.UserDAO;

@Service
public class UserDAOImpl extends GenericDAOImpl<User> implements UserDAO {
	private EntityManagerFactory entityManagerFactory;

	public void saveUser(User user) {
		entityManagerFactory = Persistence.createEntityManagerFactory("miUP");
		EntityManager em = entityManagerFactory.createEntityManager();
		EntityTransaction tx = em.getTransaction();

		try {
			tx.begin();

			if (!(user.getTransactions() == null)) {
				Set<Event> managedEvents = new HashSet<>();
				for (Event event : user.getTransactions()) {
					Event managedEvent = em.find(Event.class, event.getId());
					if (managedEvent == null) {
						managedEvent = event;
					}
					managedEvent.setUser(user);
					managedEvents.add(managedEvent);
				}
				user.setTransactions(managedEvents);
			}
			em.merge(user);
			tx.commit();
		} catch (PersistenceException e) {
			Throwable cause = e.getCause();
			while (cause != null) {
				//System.out.println("Causa: " + cause);
				if (cause instanceof SQLException) {
					SQLException sqlException = (SQLException) cause;
					int errorCode = sqlException.getErrorCode();
					String sqlState = sqlException.getSQLState();
					//System.out.println("Código de Error SQL: " + errorCode);
					//System.out.println("Estado SQL: " + sqlState);
					if (errorCode == 1062 || "23000".equals(sqlState)) {
						//System.out.println("Error: La dirección de correo electrónico ya está en uso.");
						throw new DuplicateKeyException("La dirección de correo electrónico ya está en uso.");
					} else {
						//System.out.println("Error SQL: " + errorCode + ", Estado SQL: " + sqlState);
						//System.out.println("Error al guardar el usuario: " + e.getMessage());
					}
					break; // Salir del bucle una vez que encontramos una SQLException
				}
				cause = cause.getCause(); // Obtener la causa siguiente
			}
			if (tx.isActive()) {
				tx.rollback();
			}
			e.printStackTrace();
		} catch (Exception e) {
			System.out.println("NOT");
			if (tx.isActive()) {
				tx.rollback();
			}
			e.printStackTrace();
		} finally {
			em.close();
		}
	}

}
